// eased shapes test bed
// filename: es_test.scad

/* Copyright 2018-2020 Andrew H. Wakefield

    es_test.scad is part of the eased_shapes.scad library

    This library is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

include <eased_shapes.scad>

$fn=64;

b_size = [30,20,10];
t_size = [20,15,10];
tb_type = es_none;
c_type = es_concave;
sto = false;
eh = true;
ed = true;
em = true;
c = false;
/*
eased_cylinder_cutout(r1=5,r2=1,h=20,
    top=1,top_type=es_fillet,
    bottom=2,bottom_type=es_concave);
*/

inside_corner(corner=5,h=20,
    corner_type=c_type,
    top=1,
    top_type=es_bevel,
    bottom=1,
    bottom_type=es_fillet,
    on_z=true,
    center=true
);    
//*/
//es_min_size=1;
/*
color("red")
translate([50,0,0])
eased_cube(size_bottom=b_size,
    size_top=t_size,
    slope_to_origin=sto,
    enable_hull=eh);

color("blue")
translate([0,50,0])
eased_cube(size_bottom=b_size,
    size_top=t_size,
    slope_to_origin=sto,
    enable_hull=eh);
//
translate([0,0,0])
eased_cube_cutout(size_bottom=b_size,
    size_top=t_size,
    top=1,top_type=tb_type,
    bottom=3,bottom_type=tb_type,
    corners=2,corner_type=c_type,
    slope_to_origin=sto,
    enable_hull=eh,
    enable_difference=ed,
    enable_middle=em,
    center=c);
//*/

/*
es_test_cube_cutout_tb(
    size=[40,30,20],
//    size_top=[30,20,20],
//    size_bottom=[30,20,20],
    top=1, top_type=es_bevel,
    bottom=.5, bottom_type=es_bevel,
    corners=1, corner_type=es_fillet,
    enable_difference=true,
    slope_to_origin=false
    );
//*/
module es_test_cube_cutout_tb (
    size = [-1,-1,-1],
    size_top = [-1,-1,-1],
    size_bottom = [-1,-1,-1],
    h = -1,
    height = -1,
    top = 1,
    bottom = 3,
    corners = 2,
    top_type = es_none,
    bottom_type = es_none,
    slope_to_origin = false
    )
{
    types = [ es_none, es_bevel, es_fillet, es_concave ];

    max_x = max(size[0],size_top[0],size_bottom[0]);
    max_y = max(size[1],size_top[1],size_bottom[1]);
    
    for ( i = [0:3] ) {
        x = i * (max_x + 10 );
        
        translate([x,0,0])
        eased_cube_cutout(size,size_top,size_bottom,
            h,height,
            top,bottom,corners,slope_to_origin,
            top_type = top_type,
            bottom_type = bottom_type,
            corner_type = types[i]
        );
    }
}

module es_test_cube_cutout_corner (
    size = [-1,-1,-1],
    size_top = [-1,-1,-1],
    size_bottom = [-1,-1,-1],
    h = -1,
    height = -1,
    top = 1,
    bottom = 3,
    corners = 2,
    corner_type = es_none,
    slope_to_origin = false
    )
{
    types = [ es_none, es_bevel, es_fillet, es_concave ];

    max_x = max(size[0],size_top[0],size_bottom[0]);
    max_y = max(size[1],size_top[1],size_bottom[1]);
    
    for ( i = [0:3], j = [0:3] ) {
        x = i * (max_x + 10 );
        y = j * (max_y + 10 );
        
        translate([x,y,0])
        eased_cube_cutout(size,size_top,size_bottom,
            h,height,
            top,bottom,corners,slope_to_origin,
            corner_type = corner_type,
            top_type = types[i],
            bottom_type = types[j]
        );
    }
}

module es_test_cube_corner (
    size = [-1,-1,-1],
    size_top = [-1,-1,-1],
    size_bottom = [-1,-1,-1],
    h = -1,
    height = -1,
    top = 1,
    bottom = 3,
    corners = 2,
    corner_type = es_none,
    slope_to_origin = false
    )
{
    types = [ es_none, es_bevel, es_fillet, es_concave ];

    max_x = max(size[0],size_top[0],size_bottom[0]);
    max_y = max(size[1],size_top[1],size_bottom[1]);
    
    for ( i = [0:3], j = [0:3] ) {
        x = i * (max_x + 10 );
        y = j * (max_y + 10 );
        
        translate([x,y,0])
        eased_cube(size,size_top,size_bottom,h,height,
            top,bottom,corners,slope_to_origin,
            corner_type = corner_type,
            top_type = types[i],
            bottom_type = types[j]
        );
    }
}

module es_test_cube_bottom (
    size = [-1,-1,-1],
    size_top = [-1,-1,-1],
    size_bottom = [-1,-1,-1],
    h = -1,
    height = -1,
    top = 1,
    bottom = 3,
    corners = 2,
    bottom_type = es_none,
    slope_to_origin = false
    )
{
    types = [ es_none, es_bevel, es_fillet, es_concave ];

    max_x = max(size[0],size_top[0],size_bottom[0]);
    max_y = max(size[1],size_top[1],size_bottom[1]);
    
    for ( i = [0:3], j = [0:3] ) {
        x = i * (max_x + 10 );
        y = j * (max_y + 10 );
        
        translate([x,y,0])
        eased_cube(size,size_top,size_bottom,h,height,
            top,bottom,corners,slope_to_origin,
            bottom_type = bottom_type,
            top_type = types[i],
            corner_type = types[j]
        );
    }
}

module es_test_cube_top (
    size = [-1,-1,-1],
    size_top = [-1,-1,-1],
    size_bottom = [-1,-1,-1],
    h = -1,
    height = -1,
    top = 1,
    bottom = 3,
    corners = 2,
    top_type = es_none,
    slope_to_origin = false
    )
{
    types = [ es_none, es_bevel, es_fillet, es_concave ];

    max_x = max(size[0],size_top[0],size_bottom[0]);
    max_y = max(size[1],size_top[1],size_bottom[1]);
    
    for ( i = [0:3], j = [0:3] ) {
        x = i * (max_x + 10 );
        y = j * (max_y + 10 );
        
        translate([x,y,0])
        eased_cube(size,size_top,size_bottom,h,height,
            top,bottom,corners,slope_to_origin,
            top_type = top_type,
            bottom_type = types[i],
            corner_type = types[j]
        );
    }
}

module es_test_cube_all (
    size = [-1,-1,-1],
    size_top = [-1,-1,-1],
    size_bottom = [-1,-1,-1],
    h = -1,
    height = -1,
    top = 1,
    bottom = 3,
    corners = 2,
    slope_to_origin = false
    )
{
    types = [ es_none, es_bevel, es_fillet, es_concave ];
    max_z = max(size[2],size_top[2],size_bottom[2],
                h, height);

    for ( k = [0:3] ) {
        z = k * (max_z + 10 );
        
        translate([0,0,z])
        es_test_cube_bottom(size,size_top,size_bottom,
            h,height,top,bottom,corners,slope_to_origin,
            bottom_type = types[k]);
    }
}
