// eased_shape utility routines
// filename: es_utilities.scad

/* Copyright 2018-2020 Andrew H. Wakefield

    es_utilities.scad is part of the eased_shapes.scad library

    This library is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

module rotate_copy ( v = [ 0,0,0],
                     do_copy = true,
                     do_rotate = true )
{
  if ( do_copy )
    children();

  if ( do_rotate )
    rotate(v) children();
}

module mirror_copy ( v = [1,0,0],
                     do_copy = true,
                     do_mirror = true )
{
  if ( do_copy )
    children();

  if ( do_mirror )
    mirror(v) children();
}

module skew_over_z ( x = 0, y = 0, z = 1,
    x_angle = -99, y_angle = -99, tilt = false )
{
    x_shear = x_angle > -90 ? tan(x_angle ) : x / z;
    y_shear = y_angle > -90 ? tan(y_angle) : y / z;

    rot_x = tilt ? atan(y_shear) : 0;
    rot_y = tilt ? -atan(x_shear) : 0;
    
    m = [ [ 1, 0, x_shear, 0 ],
           [ 0, 1, y_shear, 0 ],
           [ 0, 0, 1, 0 ],
           [ 0, 0, 0, 1 ] ];

    rotate([rot_x,rot_y,0])
    multmatrix(m)
    children();  
}

module skew_over_x ( x = 1, y = 0, z = 0,
    y_angle = -99, z_angle = -99, tilt = false )
{
    y_shear = y_angle > -90 ? tan(y_angle) : y / x;
    z_shear = z_angle > -90 ? tan(z_angle ) : z / x;

    rot_y = tilt ? -atan(y_shear) : 0;
    rot_z = tilt ? atan(z_shear) : 0;
    
    m = [ [ 1, 0, 0, 0 ],
           [ y_shear, 1, 0, 0 ],
           [ z_shear, 0, 1, 0 ],
           [ 0, 0, 0, 1 ] ];

    rotate([0,rot_y,rot_z])
    multmatrix(m)
    children();  
}

module skew_over_y ( x = 0, y = 1, z = 0,
    x_angle = -99, z_angle = -99, tilt = false )
{
    x_shear = x_angle > -90 ? tan(x_angle) : x / y;
    z_shear = z_angle > -90 ? tan(z_angle ) : z / y;

    rot_x = tilt ? -atan(x_shear) : 0;
    rot_z = tilt ? atan(z_shear) : 0;
    
    m = [ [ 1, x_shear, 0, 0 ],
           [ 0, 1, 0, 0 ],
           [ 0, z_shear, 1, 0 ],
           [ 0, 0, 0, 1 ] ];

    rotate([rot_x,0,rot_z])
    multmatrix(m)
    children();  
}

module es_difference_if ( b = false )
{
    if ( b && $children > 1 ) {
        difference() {
            children(0);
            children([1:$children-1]);
        }
    } else {
        union() children();
    }
}

module es_hull_if ( b = false )
{
    if ( b ) {
        hull() children();
    } else {
        union() children();
    }
}

