// Creates shapes to make holes for
// typical screws and rods used in 3d printing.
//
// Requires the eased_shapes.scad library
//
/* Copyright 2018-2020 Andrew H. Wakefield

    This library is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

sr_bevel = .5;

r_1_2 = 6.75;
r_1_2_thread = 5.9;
r_1_2_counter = 9;
r_1_2_counter_typical_depth = 10;
//r_1_2_nut_w = 16;   //???
//r_1_2_nut_r = sr_get_radius(r_3_8_nut_w);
//r_1_2_nut_typical_depth = 8;

r_3_8 = 5;
r_3_8_thread = 4.5;
r_3_8_counter = 8;
r_3_8_counter_typical_depth = 10;
r_3_8_nut_w = 14.3;
r_3_8_nut_r = sr_get_radius(r_3_8_nut_w);
r_3_8_nut_typical_depth = 8;

r_5_16 = 4.2;
r_5_16_thread = 3.8;
r_5_16_counter = 6.4;
r_5_16_counter_typical_depth = 8;
r_5_16_nut_w = 12.9;
r_5_16_nut_r = sr_get_radius(r_5_16_nut_w);
r_5_16_nut_typical_depth = 7;

r_1_4 = 3.4;
r_1_4_thread = 2.8;
r_1_4_counter = 6.4;
r_1_4_counter_typical_depth = 4.5;
r_1_4_nut_w = 5.75;
r_1_4_nut_r = sr_get_radius(r_1_4_nut_w);
r_1_4_nut_typical_depth = 5;

r_n10 = 2.6;
r_n10_thread = 2.1;
r_n10_counter = 4.9;
r_n10_counter_typical_depth = 3.5;
r_n10_nut_w = 9.8;
r_n10_nut_r = sr_get_radius(r_n10_nut_w);
r_n10_nut_typical_depth = 3.5;

r_n8 = 2.3;
r_n8_thread = 1.9;
r_n8_counter = 4.3;
r_n8_counter_typical_depth = 3;
r_n8_nut_w = 9;
r_n8_nut_r = sr_get_radius(r_n8_nut_w);
r_n8_nut_typical_depth = 3.5;

r_acme_1_5_10 = 5.4;

r19 = 9.7;
r10 = 5.3;
r9 = 4.8;
r8 = 4.3;

r5 = 2.8;
r5_counter = 4.6;
r5_counter_typical_depth = 6;
r5_thread = 2.3;
r5_nut_w = 8.3;         // across flats
r5_nut_r = sr_get_radius(r5_nut_w);
r5_nut_typical_depth = 4.7;

r3 = 1.8;
r3_counter = 3;
r3_counter_typical_depth = 2;
r3_nut_w = 5.8;                     // across flats
r3_nut_r = sr_get_radius(r3_nut_w);
r3_nut_typical_depth = 3.2;

r2 = 1.3;
r2_counter = 2.2;
r2_counter_typical_depth = 2;
r2_nut_w = 4.3;
r2_nut_r = sr_get_radius(r2_nut_w);
r2_nut_typical_depth = 2;

function sr_get_radius ( width )
    = width / ( 2 * cos(30) );

function sr_get_width ( radius )
    = radius * 2 * cos(30);

module sr_segment
  (
    r,              // radius of this segment
    h,              // height of this segment
    hex = false,    // if true, this segment is a hex
    top_flange = 0,     // if > 0, size of cutout bevel
    bottom_flange = 0, // if > 0, size of cutout bevel
    top_transition = 0, // if > 0, angle of transition
    bottom_transition = 0,  // same as above
    top_trans_r = 0,        // radius to transition to
    bottom_trans_r = 0     // radius to transition to
  )
{
  tf = top_transition > 0 ? 0 : top_flange;
  bf = bottom_transition > 0 ? 0 : bottom_flange;

  ttr = max(0,min(r,top_trans_r));
  btr = max(0,min(r,bottom_trans_r));

  tta = max(0,min(89,top_transition));
  bta = max(0,min(89,bottom_transition));

  fn = hex ? 6 : $fn;

  eased_cylinder_cutout(r=r,h=h,
    top=tf,top_type=es_bevel,
    bottom=bf,bottom_type=es_bevel,
    $fn=fn);
    
  if ( tta > 0 ) {
    tfh = ( r - ttr ) * tan( 90 - tta );
    translate([0,0,h])
    cylinder(r1=r,r2=ttr,h=tfh,$fn=fn);
  }

  if ( bta > 0 ) {
    bfh = ( r - btr ) * tan( 90 - bta );
    rotate([180,0,0])
    cylinder(r1=r,r2=btr,h=bfh,$fn=fn);
  }
}

module sr_screw
  (
    r,      // radius of the screw
    h,      // total height not including extensions/relief
    extension = 0,     // if > 0, extends threads or
                        // body below z=0
    thread_r = 0,     // thread radius
    thread_h = 0,     // length of threads
    counter_r = 0,     // countersink size
    counter_h = 0,     // height of countersink
    counter_gap = 0,  // if > 0, add support bridge
    counter_transition = 0, // if > 0, angle of transition
    counter_extension = 0,  // extra above the countersink
    nut_r = 0,         // nut size (radius of circle)
    nut_w = 0,        // nut size (width across flats)
    nut_h = 0,        // height of nut
    nut_extension = 0,// if > 0, extends nut below
    nut_t_gap = 0,    // if > 0, add support bridge
    nut_b_gap = 0,   // if > 0, add support bridge
    nut_t_transition = -1, // if > 0, angle of top trans
    nut_b_transition = -1, // if > 0, angle of b trans
    nut_transition = 0, // if > 0, angle of t/b trans
    nut_position = 0,  // if > 0, moves nut up
    outlet = 0,	      // if > 0, create a side rectangle
                      // into which nut can be inserted
    rot = 0,		      // rotation of nut and outlet
                      // where 0 = east, 90 = north, etc.
    bevels = 0,       // size of bevels
    center = false     // if true, center vertically
  )
{
  // thread dimensions - zeroed if both are not set

  thr_r = thread_h > 0 ? max(0,thread_r) : 0;
  thr_h = thr_r > 0 ? max(0,thread_h) : 0;

  // countersink dimensions - zeroed as appropriate

  cr = ( counter_h > 0 || counter_extension > 0 ) ?
       max(0,counter_r) : 0;
  ch = cr > 0 ? max(0,counter_h) : 0;
  cx = cr > 0 ? max(0,counter_extension) : 0;

  // countersink transition angle

  cta = cr > 0 && ch > 0 ?
    max(0,min(89,counter_transition)) : 0;

  // countersink gap

  cg = cr > 0 && ch > 0 && cta == 0 ?
    max(0,counter_gap) : 0;

  // nut dimensions; again, zeroed unless r & h both set

  nr = nut_w > 0 ? sr_get_radius(nut_w) : max(0,nut_r);
  nh = nr > 0 ? max(0,nut_h) : 0;
  np = nr > 0 && nh > 0 ? max(0,nut_position) : 0;

  // nut extension only valid if nut at z=0

  n_ext = nr > 0 && np == 0 ? max(0,nut_extension) : 0;

  // bottom nut flare is valid only if nut is at
  // the bottom with no extension

  nf = ( n_ext == 0 && np == 0 ) ? bevels : 0;

  // nut transition angles above and below, melded
  // with nut_transition default value

  nta = nut_t_transition >= 0 ? nut_t_transition
                              : nut_transition;
  nba = nut_b_transition >= 0 ? nut_b_transition
                              : nut_transition;

  // nut transition angles only valid if r & h are set
  // buttom nut transition only valid if no extension

  ntta = nr > 0 && nh > 0 ? max(0,min(89,nta)) : 0;
  nbta = nr > 0 && nh > 0 && n_ext == 0 ?
    max(0,min(89,nba)) : 0;

  // nut gaps only valid if r & h set and no transition

  ntg = nr > 0 && nh > 0 && ntta == 0 ? max(0,nut_t_gap)
                                      : 0;

  // nut bottom gap only valid if the nut position > 0

  nbg = nr > 0 && nh > 0 && np > 0 && nbta == 0 ?
    max(0,nut_b_gap) : 0;

  // center screw on the h value

  zt = center ? -h/2 : 0;

  // npg is the maximum of 0 or the nut position
  // less any bottom gap

  npg = max(0,np-nbg);

  // vectors to control the dimensions of
  // any segments below the nut but above z=0.
  // Members are [ radius, height, top flange ]
 
  // seg1 represents a threaded section below the
  // nut, but above z=0, if any
  //
  // Note that np/npg can only be greater than 0
  // if nr > 0 && nh > 0; note that seg1r, seg1h,
  // seg1f will be invalid if npg = 0, but if so
  // they will be discarded when the seg1 vector
  // is created. Νote that seg1[1] will be 0 if
  // npg = 0; otherwise seg1[1] will be <= npg.

  seg1r = thr_r > 0 ? thr_r : r;
  seg1h = thr_h > 0 ? min(npg,thr_h) : min(npg,h);
  seg1f = seg1h < npg ? r - thr_r
        : seg1h == npg && nbta <= 0 ? bevels 
        : 0;

  seg1 = npg > 0 ? [ seg1r, seg1h, seg1f ]
       : [ 0, 0, 0 ];

  // seg2 represents the unthreaded portion below
  // the nut, but above z=0, if any
  //
  // seg2 will only be needed if there is a threaded
  // portion (thr_r > 0) that has already been
  // used up in seg1, still leaving more below
  // the nut that needs to be filled with the body
  // portion (r); otherwise seg2 will be [0,0,0].
  //
  // Once again, seg2r, seg2h, seg2f will be
  // invalid if npg = 0, but if so they will be
  // discarded when the seg2 vector is created.

  seg2r = r;
  seg2h = npg - seg1[1];
  seg2f = ( nbta <= 0 ) ? bevels : 0;

  seg2 = npg > seg1[1] && thr_r > 0 ?
    [ seg2r, seg2h, seg2f ] : [ 0, 0, 0 ];

  // calculate the dimensions for any extension
  // of the threads/body that go below z=0.
  // It will always be bevelled on the bottom;
  // it will have a flange on top only if np=0 and
  // n_ext = 0

  ext_h = max(0,extension);
  ext_r = ext_h > 0 ? thr_r > 0 ? thr_r : r : 0;
  ext_f = ( npg == 0 && n_ext == 0 ) ? bevels : 0;

  translate([0,0,zt])
  union() {
    // first put in any extension below z=0

    if ( ext_h > 0 ) {
      translate([0,0,-ext_h])
      sr_segment(r=ext_r,h=ext_h,
        top_flange=ext_f,
        bottom_transition=45,
        bottom_trans_r=ext_r-bevels);
    }

    // the next two segments only occur when
    // the nut (and any gap) is positioned above
    // z=0. Most dimensions are calculated above,
    // except for the bottom flange on seg1; this
    // is added only if there is no body extension.

    if ( seg1[0] > 0 ) {
      bf = ext_h > 0 ? 0 : bevels;

      sr_segment(r=seg1[0],h=seg1[1],
        top_flange=seg1[2],
        bottom_flange=bf);
    }

    if ( seg2[0] > 0 ) {
      translate([0,0,seg1[1]])
      sr_segment(r=seg2[0],h=seg2[1],
        top_flange=seg2[2]);
    }

    // nut comes next; note that by definition
    // any nut extension extends below z=0 and
    // requires nut position to be at z=0. To
    // say this another way, either np OR n_ext
    // (or both) MUST equal zero

    if ( nr > 0 && nh > 0 ) {
      bf = ( np == 0 && n_ext == 0
        && ext_h == 0 ) ? bevels : 0;
      btr = seg2[0] > 0 ? seg2[0]
          : seg1[0] > 0 ? seg1[0]
          : 0;
      ttr = thr_r > np + nh ? thr_r : r;

      rotate([0,0,rot])
      translate([0,0,np-n_ext])
      union() {
        sr_segment(r=nr,h=nh+n_ext,hex=true,
          bottom_flange=bf,
          bottom_transition=nbta,
          bottom_trans_r = btr,
          top_transition=ntta,
          top_trans_r = ttr);
              
        // if outlet > 0, it describes an "outlet"
        // that extends the nut space horizontally

        if ( outlet > 0 ) {
          rotate([0,90,0])
          translate([-nh,0,0])
          eased_cube_cutout(
            [nh,sr_get_width(nr),outlet],
            top = bevels, top_type = es_bevel,
            center_y = true );
        }
      }
    }

    // threads formed next; normally either threads
    // or nut, but not both - but both can be
    // done if desired. Note that thr_r will be 0
    // if thread_h <= 0, and likewise thr_h
    // will be 0 if thread_r <= 0. Threads are
    // assumed to begin at z=0, but actual
    // segment begins above the nut (if any).
    // The segment needs to include a flange
    // at the bottom of the hole or at the nut
    // if the nut does not include a transition. If
    // thr_h is not long enough to extend beyond
    // thenut, no threads will be created.

    // th_z will be 0 if there is no nut;
    // th_h will be the remaining height
    // of the threads up to any countersink (and
    // countersink gap, if any).

    th_z = max(0, np + nh + ntg);
    th_h = min(max(0, h - th_z - ch - cg),
               max(0, thr_h - th_z));

    // body_z is the top of all segments after the
    // threaded section (if any) is generated

    body_z = th_z + th_h;

    // th_bf will be 0 if there is a nut and it has
    // a top transition; otherwise it will = bevels;
    // th_tf will be r-thr_r if the thread connects
    // to a screw clearance; 0 if it connects
    // to a countersink with transition;
    // otherwise = bevels.

    th_bf = ntta > 0 ? 0 : bevels;
    th_tf = body_z < h - ch - cg ? r - thr_r
          : cta > 0 ? 0
          : bevels;

    if ( thr_r > 0 && th_h > 0 ) {
      translate([0,0,th_z])
      sr_segment(r=thr_r,h=th_h,
        bottom_flange=th_bf,
        top_flange=th_tf);
    }

    // body of screw comes next - if there is
    // any length left to do

    body_h = h - ch - cg - body_z;
    bf = body_z == th_z ? bevels : 0;
    tf = cta > 0 ? 0 : bevels;

    if ( body_h > 0 ) {
      translate([0,0,body_z])
      sr_segment(r=r,h=body_h,
        top_flange=tf,
        bottom_flange=bf);
    }

    // next, the countersink, if any

    if ( cr > 0 && ch > 0 ) {
      btr = body_h > 0 ? r : thr_r;

      translate([0,0,h-ch])
      sr_segment(r=cr,h=ch,
        top_flange=bevels,
        bottom_transition=cta,
        bottom_trans_r=btr);
    }
    
    // finally, the countersink extension, if any
    
    if ( cr > 0 && cx > 0 ) {
      r = ( ch > 0 ) ? cr + bevels : cr;
      
      translate([0,0,h])
      sr_segment(r=r,h=cx);
    }
  }
}

module m2
  (
    r = r2,
    h = 10,  // total height not including extra relief
    extension = 0,
    counter_r = r2_counter,
    counter_h = r2_counter_typical_depth,
    counter_gap = 0,  // if > 0, allows for support web
    counter_transition = 60, // reset to 0 to enable gap
    counter_extension = 0, // if > 0, extend counter above
    nut_r = 0,
    nut_w = 0,
    nut_h = r2_nut_typical_depth,
    nut_extension = 0,  // if > 0, extends nut below
    nut_t_gap = 0,     // if > 0, allows for support web
    nut_b_gap = 0,
    nut_transition = 0,
    nut_t_transition = 60, // reset to 0 to enable gap
    nut_b_transition = 0,
    nut_position = 0,
    outlet = 0,
    rot = 0,
    bevels = sr_bevel,
    center = false
  )
{
  nr = nut_w > 0 ? sr_get_radius(nut_w) : nut_r;

  sr_screw(r=r,h=h,
    extension=extension,
    counter_r=counter_r,
    counter_h=counter_h,
    counter_gap=counter_gap,
    counter_transition=counter_transition,
    counter_extension = counter_extension,
    nut_r = nr > 0 ? nr : r2_nut_r,
    nut_h = nut_h,
    nut_extension = nut_extension,
    nut_t_gap = nut_t_gap,
    nut_b_gap = nut_b_gap,
    nut_transition = nut_transition,
    nut_t_transition = nut_t_transition,
    nut_b_transition = nut_b_transition,
    nut_position = nut_position,
    outlet = outlet,
    rot = rot,
    bevels = bevels,
    center = center);
}

module m3
  (
    r = r3,
    h = 10,  // total height not including extra relief
    extension = 0,
    counter_r = r3_counter,
    counter_h = r3_counter_typical_depth,
    counter_gap = 0,  // if > 0, allows for support web
    counter_transition = 60, // reset to 0 to enable gap
    counter_extension = 0, // if > 0, extend counter above
    nut_r = 0,
    nut_w = 0,
    nut_h = r3_nut_typical_depth,
    nut_extension = 0,  // if > 0, extends nut below
    nut_t_gap = 0,     // if > 0, allows for support web
    nut_b_gap = 0,
    nut_transition = 0,
    nut_t_transition = 60, // reset to 0 to enable gap
    nut_b_transition = 0,
    nut_position = 0,
    outlet = 0,
    rot = 0,
    bevels = sr_bevel,
    center = false
  )
{
  nr = nut_w > 0 ? sr_get_radius(nut_w) : nut_r;

  sr_screw(r=r,h=h,
    extension=extension,
    counter_r=counter_r,
    counter_h=counter_h,
    counter_gap=counter_gap,
    counter_transition=counter_transition,
    counter_extension = counter_extension,
    nut_r = nr > 0 ? nr : r3_nut_r,
    nut_h = nut_h,
    nut_extension = nut_extension,
    nut_t_gap = nut_t_gap,
    nut_b_gap = nut_b_gap,
    nut_transition = nut_transition,
    nut_t_transition = nut_t_transition,
    nut_b_transition = nut_b_transition,
    nut_position = nut_position,
    outlet = outlet,
    rot = rot,
    bevels = bevels,
    center = center);
}

module m5
  (
    r = r5,
    h = 10,            // total height
    extension = 0,
    thread_r = r5_thread,
    thread_h = 0,     // height of threads
    counter_r = r5_counter,
    counter_h = r5_counter_typical_depth,
    counter_gap = 0,  // if > 0, allows for support web
    counter_transition = 60, // reset to 0 to enable gap
    counter_extension = 0, // if > 0, extend counter above
    nut_r = 0,
    nut_w = 0,
    nut_h = r3_nut_typical_depth,
    nut_extension = 0,  // if > 0, extends nut below
    nut_t_gap = 0,     // if > 0, allows for support web
    nut_b_gap = 0,
    nut_transition = 0,
    nut_t_transition = 60, // reset to 0 to enable gap
    nut_b_transition = 0,
    nut_position = 0,
    outlet = 0,
    rot = 0,
    bevels = sr_bevel,
    center = false
  )
{
  nr = nut_w > 0 ? sr_get_radius(nut_w) : nut_r;

  sr_screw(r=r,h=h,
    extension=extension,
    thread_r=thread_r,
    thread_h=thread_h,
    counter_r=counter_r,
    counter_h=counter_h,
    counter_gap = counter_gap,
    counter_transition = counter_transition,
    counter_extension = counter_extension,
    nut_r = nr > 0 ? nr : r5_nut_r,
    nut_h = nut_h,
    nut_extension = nut_extension,
    nut_t_gap = nut_t_gap,
    nut_b_gap = nut_b_gap,
    nut_transition = nut_transition,
    nut_t_transition = nut_t_transition,
    nut_b_transition = nut_b_transition,
    nut_position = nut_position,
    outlet = outlet,
    rot = rot,
    bevels = bevels,
    center = center);
}

module sae_no_8
  (
    h = 10,            // total height
    counter_h = r_n8_counter_typical_depth,
    counter_gap = 0,  // if > 0, allows for support web
    counter_transition = 0,  // angle of transition
    counter_extension = 0, // if > 0, extend counter above
    thread_h = 0,     // height of threads
    bevels = sr_bevel,
    center = false
  )
{
  sr_screw(r=r_n8,h=h,
    counter_r=r_n8_counter,
    counter_h=counter_h,
    counter_gap = counter_gap,
    counter_transition = counter_transition,
    counter_extension = counter_extension,
    thread_r = r_n8_thread,
    thread_h = thread_h,
    bevels = bevels,
    center = center);
}

module sae_no_10
  (
    h = 10,            // total height
    counter_h = r_n10_counter_typical_depth,
    counter_gap = 0,  // if > 0, allows for support web
    counter_transition = 0, // if > 0, angle of transition
    counter_extension = 0, // if > 0, extend counter above
    thread_h = 0,     // height of threads
    bevels = sr_bevel,
    center = false
  )
{
  sr_screw(r=r_n8,h=h,
    counter_r=r_n10_counter,
    counter_h=counter_h,
    counter_gap = counter_gap,
    counter_transition = counter_transition,
    counter_extension = counter_extension,
    thread_r = r_n10_thread,
    thread_h = thread_h,
    bevels = bevels,
    center = center );
}

module sae_1_4
  (
    h = 10,            // total height
    counter_h = r_1_4_counter_typical_depth,
    counter_r = r_1_4_counter,
    counter_gap = 0,  // if > 0, allows for support web
    counter_transition = 0, // if > 0, angle of transition
    counter_extension = 0, // if > 0, extend counter above
    thread_h = 0,     // height of threads
    bevels = sr_bevel,
    center = false
  )
{
  sr_screw(r=r_1_4,h=h,
    counter_r=counter_r,
    counter_h=counter_h,
    counter_gap = counter_gap,
    counter_transition = counter_transition,
    counter_extension = counter_extension,
    thread_r = r_1_4_thread,
    thread_h = thread_h,
    bevels = bevels,
    center = center);
}

module sae_5_16
  (
    h = 10,            // total height
    counter_h = r_5_16_counter_typical_depth,
    counter_gap = 0,  // if > 0, allows for support web
    counter_transition = 0, // if > 0, angle of transition
    counter_extension = 0, // if > 0, extend counter above
    thread_h = 0,     // height of threads
    bevels = sr_bevel,
    center = false
  )
{
  sr_screw(r=r_5_16,h=h,
    counter_r=r_5_16_counter,
    counter_h=counter_h,
    counter_gap = counter_gap,
    counter_transition = counter_transition,
    counter_extension = counter_extension,
    thread_r = r_5_16_thread,
    thread_h = thread_h,
    bevels = bevels,
    center = center);
}

module sae_3_8
  (
    h = 10,            // total height
    counter_h = r_3_8_counter_typical_depth,
    counter_gap = 0,  // if > 0, allows for support web
    counter_transition = 0, // if > 0, angle of transition
    counter_extension = 0, // if > 0, extend counter above
    thread_h = 0,     // height of threads
    bevels = sr_bevel,
    center = false
  )
{
  sr_screw(r=r_3_8,h=h,
    counter_r=r_3_8_counter,
    counter_h=counter_h,
    counter_gap = counter_gap,
    counter_transition = counter_transition,
    counter_extension = counter_extension,
    thread_r = r_3_8_thread,
    thread_h = thread_h,
    bevels = bevels,
    center = center);
}

module sae_1_2
  (
    h = 10,            // total height
    counter_h = r_1_2_counter_typical_depth,
    counter_r = r_1_2_counter,
    counter_gap = 0,  // if > 0, allows for support web
    counter_transition = 0, // if > 0, angle of transition
    counter_extension = 0, // if > 0, extend counter above
    thread_h = 0,     // height of threads
    bevels = sr_bevel,
    center = false
  )
{
  sr_screw(r=r_1_2,h=h,
    counter_r=counter_r,
    counter_h=counter_h,
    counter_gap = counter_gap,
    counter_transition = counter_transition,
    counter_extension = counter_extension,
    thread_r = r_1_2_thread,
    thread_h = thread_h,
    bevels = bevels,
    center = center);
}
